/* 
 * i-scream central monitoring system
 * http://www.i-scream.org.uk
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.core;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * The main class for the CORE of the I-Scream system.
 * This class is the component for CORE I-Scream services
 * namely the Logger and the ConfigurationManager.
 * It registers and starts them, at which point it essentially
 * finishs its role and the started services continue to
 * serve requests from the I-Scream system.
 *
 * @author  $Author: tdb $
 * @version $Id: Core.java,v 1.28 2003/02/05 16:43:47 tdb Exp $
 */
public class Core implements Component {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.28 $";
    
    /**
     * The friendly name for this component, used by
     * all related classes.
     */
    public static final String NAME = "Core";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---

    /**
     * This method starts the CORE
     * Currently the args are passed direct to the ORB,
     * so any ORB paramaters could go there.
     *
     */
    public void start() throws ComponentStartException {
        // start and bind each server in turn
        
        // work out which logger to use
        String whichLogger = System.getProperty("uk.org.iscream.cms.server.LoggerClass");
        String loggerPackage = System.getProperty("uk.org.iscream.cms.server.LoggerPackage");
       
        // construct the relevant LoggerImpl
        LoggerImpl loggerImplRef = null;
        try {
            loggerImplRef = (LoggerImpl) ClassLoader.getSystemClassLoader().loadClass(loggerPackage + "." + whichLogger).newInstance();
        } catch (Exception e) {
            // if anything goes wrong we throw a failed start exception
            throw new ComponentStartException("unable to load logging class");
        }

        // setup and bind the LoggerServant
        LoggerServant loggerRef = new LoggerServant(loggerImplRef);
        _refman.bindToOrb(loggerRef, "iscream.Logger");

        // get a reference to the servant as a Logger
        _logger = _refman.getLogger();	        
    
        // create the Configurator
        ConfigurationManagerServant configManRef = new ConfigurationManagerServant();
        
        // and advertise it to the naming context
        _refman.bindToOrb(configManRef, "iscream.ConfigurationManager");

        _logger.write(toString(), Logger.SYSINIT, "started");
    }
    
    /**
     * Does a dependency check. Used mainly at startup to
     * see if the required dependencies (components) are up
     * and running.
     *
     * @return a boolean value, true if the depdencies are satisfied
     */
    public boolean depCheck() {
        // the core has no depedencies
        return true;
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = Core.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger;

    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
//---STATIC ATTRIBUTES---

}
